@@ -4,6 +4,8 @@ from cro_validate.enum import DataType
 
 
 def get(definition_or_name):
+	'''returns :ref:`Definition` instance.
+	'''
 	return Definitions.Index.get(definition_or_name)
 
 
@@ -15,6 +17,14 @@ def as_dict():
 	return Definitions.Index.as_dict()
 
 
+def to_json_dict():
+	return Definitions.Index.to_json_dict()
+
+
+def from_json_dict(root):
+	return Definitions.Index.from_json_dict(root)
+
+
 def register_definition(
  			name=None,
 			aliases=set(),
@@ -91,4 +101,8 @@ def validate_inputs(validated, **kw):
 
 def validate_input(definition_or_name, value, validated=None, field_fqn=None, field_name=None, dependent_values={}):
 	results = Definitions.Index.validate_input(validated, field_fqn, field_name, definition_or_name, value, dependent_values)
-	return results
\ No newline at end of file
+	return results
+
+
+def clear():
+	Definitions.Index.clear()
\ No newline at end of file
