@@ -1,3 +1,6 @@
+import importlib
+
+from enum import Enum
 from cro_validate.enum import DataType
 from cro_validate.classes.configuration_classes import Config
 import cro_validate.classes.parameter_classes as Parameters
@@ -10,6 +13,9 @@ class Meta:
 	def initialize(self, definition, **kw):
 		raise NotImplementedError()
 
+	def to_json_dict(self):
+		return {}
+
 
 class DefaultMeta(Meta):
 	def __init__(self, component_name_strategy=NameStrategies.DefaultComponentNameStrategy()):
@@ -26,8 +32,63 @@ class DefaultMeta(Meta):
 		else:
 			self.component_name = self.component_name_strategy.create_name(definition, component_name_suffix, display_name)
 
+		def to_json_dict(self):
+			return {
+				'component_name_strategy': self.component_name_strategy.to_json_dict()
+			}
+
 
 class Definition:
+	'''Def
+	'''
+
+	def to_json_dict(self):
+		default_value = self.default_value
+		if self.has_default_value() is False:
+			default_value = None
+		default_value_type = Utils.ClassName.class_fqn(self.default_value)
+		meta = None
+		if self.meta is not None:
+			meta = self.meta.to_json_dict()
+		meta_type = Utils.ClassName.class_fqn(self.meta)
+		dependency_resolver = None
+		if self.dependency_resolver is not None:
+			dependency_resolver = self.dependency_resolver.to_json_dict()
+		dependency_resolver_type = Utils.ClassName.class_fqn(self.dependency_resolver)
+		fmt = None
+		if self.data_format is not None:
+			if isinstance(self.data_format, Enum):
+				fmt = self.data_format.name
+			elif isinstance(self.data_format, Schemas.Schema):
+				fmt = self.data_format.to_json_dict()
+			elif isinstance(self.data_format, set):
+				fmt = [f for f in self.data_format]
+				fmt.sort()
+			else:
+				fmt = self.data_format
+		fmt_type = Utils.ClassName.class_fqn(self.data_format)
+		aliases = [a for a in self.aliases]
+		aliases.sort()
+		return {
+				#'name': self.name,
+				'aliases': aliases,
+				'description': self.description,
+				'data_type': self.data_type.name,
+				'data_format': fmt,
+				'data_format_type': fmt_type,
+				'default_value': default_value,
+				'default_value_type': default_value_type,
+				'examples': self.examples,
+				'nullable': self.nullable,
+				'deprecated': self.deprecated,
+				'is_internal': self.is_internal,
+				'rules': [{'type':Utils.ClassName.class_fqn(r), 'config':r.to_json_dict()} for r in self.rules],
+				'meta': meta,
+				'meta_type': meta_type,
+				'dependency_resolver': dependency_resolver,
+				'dependency_resolver_type': dependency_resolver_type
+			}
+
 	def __init__(
 				self,
 				name,
@@ -65,6 +126,10 @@ class Definition:
 			ConfigApi.exception_factory.create_input_error(
 					'<unset>', 'Definition name cannot be None (description={0})'.format(self.description)
 				)
+		# Aliases
+		#########
+		if isinstance(aliases, str):
+			self.aliases = {aliases}
 		# Nullable
 		##########
 		if self.default_value is None:
@@ -76,7 +141,10 @@ class Definition:
 				self.default_value = None
 		# Data Format
 		#############
-		if self.data_type is DataType.OneOf:
+		if self.data_type is DataType.Object and isinstance(self.data_format, str):
+			format_definition = Index.get(self.data_format)
+			self.data_format = format_definition.data_format
+		elif self.data_type is DataType.OneOf:
 			self.data_format = dependency_resolver.list_dependent_definition_names(self.name)
 		# Validator
 		###########
@@ -89,7 +157,6 @@ class Definition:
 		# Dependencies
 		##############
 		self.dependencies = set(self.dependency_resolver.list_dependency_fields(self.name))
-
 		# Examples
 		##########
 		if not self.examples:
@@ -122,10 +189,16 @@ class Definition:
 		model = self.data_format.model
 		if model is None:
 			raise Config.exception_factory.create_internal_error(self.data_format.model_name, 'Missing model')
-		for name in dir(model):
-			if name.startswith('_'):
-				continue
-			field_definition = getattr(model, name)
+		fields = {}
+		if isinstance(model, dict):
+			fields = model
+		else:
+			for name in dir(model):
+				if name.startswith('_'):
+					continue
+				fields[name] = getattr(model, name)
+		for name in fields:
+			field_definition = fields[name]
 			if field_definition is None:
 				field_definition = Schemas.Field()
 			if field_definition.required:
@@ -184,6 +257,9 @@ class Definition:
 		results[field_name] = value
 
 	def validate(self, results, field_fqn, field_name, definition, value, dependent_values):
+		'''
+		The validate func
+		'''
 		try:
 			if not self.validator:
 				raise Config.exception_factory.create_internal_error(self.name, "Missing validator.")
@@ -279,19 +355,51 @@ class DefaultResolver(DependentDefinitionResolver):
 	def list_dependent_definition_names(self, fqn):
 		return []
 
+	def to_json_dict(self):
+		return {}
+
 	def get_dependent_definition(self, fqn, dependent_values):
 		return None
 
 	def list_dependency_fields(self, fqn):
 		return []
 
+	def to_json_dict(self):
+		return {}
+
+
+class _OneOfResolverPlayback:
+	def __init__(self, fqn, dependency_state, dependent_definition_name):
+		self.fqn = fqn
+		self.dependency_state = dependency_state
+		self.dependent_definition_name = dependent_definition_name
+
+	def to_json_dict(self):
+		return {
+				'fqn': self.fqn,
+				'dependency_state': self.dependency_state,
+				'dependent_definition_name': self.dependent_definition_name
+			}
+
 
 class OneOfResolver(DependentDefinitionResolver):
-	def __init__(self):
+	def __init__(self, playback=[]):
 		self._dependencies = set()
 		self._dependency_idx = {}
 		self._dependency_order = []
 		self._permutations = []
+		self._serialize_playback = []
+		for entry in playback:
+			self.index_dependent_definition(
+					entry['fqn'],
+					entry['dependency_state'],
+					entry['dependent_definition_name']
+				)
+
+	def to_json_dict(self):
+		return {
+				'playback': [entry.to_json_dict() for entry in self._serialize_playback]
+			}
 
 	def _update_dependency_idx_order(self, fqn, keys):
 		diff = self._dependencies.difference(keys)
@@ -348,6 +456,7 @@ class OneOfResolver(DependentDefinitionResolver):
 	def index_dependent_definition(self, fqn, dependency_state, dependent_definition_name):
 		self._update_dependency_idx_order(fqn, dependency_state.keys())
 		self._index_permutation(fqn, dependency_state, dependent_definition_name)
+		self._serialize_playback.append(_OneOfResolverPlayback(fqn, dependency_state, dependent_definition_name))
 
 	def get_dependent_definition(self, fqn, dependent_values):
 		name = self._get_permutation_value(fqn, dependent_values)
@@ -357,6 +466,121 @@ class OneOfResolver(DependentDefinitionResolver):
 		return self._dependencies
 
 
+class DefinitionJsonDeserializer:
+	def _get_dict_value(self, idx, k, default_value=Utils.DataDefinitionDefaultValue()):
+		if k in idx:
+			return idx[k]
+		return default_value
+
+	def _set(self, src, tgt, k, default_value=Utils.DataDefinitionDefaultValue()):
+			v = self._get_dict_value(src, k, default_value)
+			if isinstance(v, Utils.DataDefinitionDefaultValue):
+				return
+			tgt[k] = v
+
+	def _deserialize_schema_field(self, root, name, obj):
+		kw = {}
+		if obj['default_value_type'] != 'cro_validate.classes.util_classes.DataDefinitionDefaultValue':
+			kw['default_value'] = self._instantiate(obj['default_value_type'], obj['default_value'])
+		else:
+			kw['default_value'] = Utils.DataDefinitionDefaultValue()
+		kw['definition_name'] = obj['definition_name']
+		if kw['definition_name'] is None:
+			kw['definition_name'] = name
+		if not Index.exists(kw['definition_name']):
+			dependent_definition_profile = self.deserialize(root, kw['definition_name'])
+			Index.register_definition(**dependent_definition_profile)
+		self._set(obj, kw, 'dependencies')
+		self._set(obj, kw, 'ignored')
+		self._set(obj, kw, 'input_name')
+		self._set(obj, kw, 'output_name')
+		self._set(obj, kw, 'required')
+		self._set(obj, kw, 'unvalidated')
+		if isinstance(kw['dependencies'], list):
+			kw['dependencies'] = set(kw['dependencies'])
+		field = Schemas.Field(**kw)
+		return field
+		
+	def _deserialize_schema(self, root, obj):
+		kw = {}
+		self._set(obj, kw, 'allow_unknown_fields')
+		self._set(obj, kw, 'case_sensitive')
+		self._set(obj, kw, 'display_name')
+		self._set(obj, kw, 'model_name')
+		self._set(obj, kw, 'model')
+		model = {}
+		for name in obj['model']:
+			field = self._deserialize_schema_field(root, name, obj['model'][name])
+			model[name] = field
+		kw['model'] = model
+		schema = Schemas.Schema(**kw)
+		return schema
+
+	def _instantiate(self, fqn, *args, **kw):
+		module_name, class_name = Utils.ClassName.class_fqn_parts(fqn)
+		if module_name == 'builtins' and class_name == 'NoneType':
+			return None
+		module = importlib.import_module(module_name)
+		_class = getattr(module, class_name)
+		if isinstance(_class, type(Enum)):
+			result = _class[args[0]]
+		else:
+			result = _class(*args, **kw)
+		return result
+
+	def _deserialize_rule(self, root, obj):
+		rule = self._instantiate(obj['type'], **obj['config'])
+		return rule
+
+	def deserialize(self, root, name):
+		kw = {}
+		namespace = {k:k for k in root}
+		for k in root:
+			for k1 in root[k]['aliases']:
+				namespace[k1] = k
+		obj = root[namespace[name]]
+		# Data Format
+		#############
+		kw['data_type'] = DataType[obj['data_type']]
+		if obj['data_format_type'] == 'cro_validate.classes.schema_classes.Schema':
+			kw['data_format'] = self._deserialize_schema(root, obj['data_format'])
+		else:
+			kw['data_format'] = self._instantiate(obj['data_format_type'], obj['data_format'])
+		# Default Value
+		###############
+		if obj['default_value_type'] == 'builtins.NoneType':
+			kw['default_value'] = None
+		elif obj['default_value_type'] != 'cro_validate.classes.util_classes.DataDefinitionDefaultValue':
+			kw['default_value'] = self._instantiate(obj['default_value_type'], obj['default_value'])
+		else:
+			kw['default_value'] = Utils.DataDefinitionDefaultValue()
+		# Meta
+		######
+		kw['meta'] = self._instantiate(obj['meta_type'], **obj['meta'])
+		# Resolver
+		##########
+		kw['dependency_resolver'] = self._instantiate(obj['dependency_resolver_type'], **obj['dependency_resolver'])
+		# Rules
+		#######
+		rules = []
+		for rule in obj['rules']:
+			rule_obj = self._deserialize_rule(obj, rule)
+			rules.append(rule_obj)
+		kw['rules'] = rules
+		# Aliases
+		#########
+		kw['aliases'] = set(obj['aliases'])
+		# Simple
+		########
+		kw['name'] = name
+		self._set(obj, kw, 'description', '')
+		self._set(obj, kw, 'examples', [])
+		self._set(obj, kw, 'nullable', False)
+		self._set(obj, kw, 'deprecated', False)
+		self._set(obj, kw, 'is_internal', False)
+		return kw
+
+
 class Index:
 	_idx = {}
 
@@ -378,6 +602,24 @@ class Index:
 	def as_dict():
 		return Index._idx
 
+	def to_json_dict():
+		aliases = set()
+		keys = [k for k in Index.as_dict()]
+		keys.sort()
+		for k in keys:
+			definition = Index.get(k)
+			aliases.update(definition.aliases)
+		result = {k:Index.get(k).to_json_dict() for k in keys if k not in aliases}
+		return result
+
+	def from_json_dict(root):
+		deserializer = DefinitionJsonDeserializer()
+		for k in root:
+			if Index.exists(k):
+				continue
+			profile = deserializer.deserialize(root, k)
+			Index.register_definition(**profile)
+
 	def register_definition(
 				name,
 				aliases,
@@ -457,4 +699,7 @@ class Index:
 		definition = Index.get(name)
 		if definition.data_type == DataType.Object:
 			return definition.validator.list_field_names()
-		return [name]
\ No newline at end of file
+		return [name]
+
+	def clear():
+		Index._idx = {}
\ No newline at end of file
