@@ -10,20 +10,18 @@ class Rule:
 				self,
 				description,
 				callback=None,
-				boundaries=[],
 				config={}
 			):
 		self._description = description
 		self._callback = callback
 		self._config = config
-		self._boundaries = list(boundaries)
+
+	def to_json_dict(self):
+		return self.get_config()
 
 	def get_config(self):
 		return self._config
 
-	def get_boundaries(self):
-		return self._boundaries
-
 	def get_description(self):
 		return self._description
 
@@ -41,7 +39,7 @@ class Noop(Rule):
 	def _noop(self, input_name, value):
 		return value
 
-	def __init__(self, description):
+	def __init__(self, description=''):
 		super().__init__(
 				description=description,
 				callback=self._noop
@@ -77,70 +75,60 @@ class AsInt(Rule):
 
 
 class IntGte(Rule):
-	def __init__(self, min_value):
+	def __init__(self, minimum):
 		super().__init__(
-				description='Must be integer greater than or equal to {0}.'.format(min_value),
+				description='Must be integer greater than or equal to {0}.'.format(minimum),
 				callback=Normalize.as_int_greater_than_or_equal_to,
-				config={'minimum':min_value}
+				config={'minimum':minimum}
 			)
-		self._boundaries.append(min_value-1)
 
 
 class IntLte(Rule):
-	def __init__(self, max_value):
+	def __init__(self, maximum):
 		super().__init__(
-				description='Must be integer less than or equal to {0}.'.format(max_value),
+				description='Must be integer less than or equal to {0}.'.format(maximum),
 				callback=Normalize.as_int_less_than_or_equal_to,
-				config={'maximum':max_value}
+				config={'maximum':maximum}
 			)
-		self._boundaries.append(max_value+1)
 
 
 class FloatGte(Rule):
-	def __init__(self, min_value):
+	def __init__(self, minimum):
 		super().__init__(
-				description='Must be float greater than or equal to {0}.'.format(min_value),
+				description='Must be float greater than or equal to {0}.'.format(minimum),
 				callback=Normalize.as_float_greater_than_or_equal_to,
-				config={'minimum':min_value}
+				config={'minimum':minimum}
 			)
-		self._boundaries.append(min_value-0.1)
 
 
 class FloatLte(Rule):
-	def __init__(self, max_value):
+	def __init__(self, maximum):
 		super().__init__(
-				description='Must be float less than or equal to {0}.'.format(max_value),
+				description='Must be float less than or equal to {0}.'.format(maximum),
 				callback=Normalize.as_float_less_than_or_equal_to,
-				config={'maximum':max_value}
+				config={'maximum':maximum}
 			)
-		self._boundaries.append(max_value+0.1)
 
 
 class FloatLt(Rule):
-	def __init__(self, max_value):
+	def __init__(self, maximum):
 		super().__init__(
-				description='Must be float less than {0}.'.format(max_value),
+				description='Must be float less than {0}.'.format(maximum),
 				callback=Normalize.as_float_less_than,
-				config={'maximum':max_value}
+				config={'maximum':maximum}
 			)
-		self._boundaries.append(max_value)
 
 
 ########################################################################################################################
 #                                                       Strings                                                        #
 ########################################################################################################################
-def _gen_str(count):
-	s = ''
-	for i in range(count):
-		s = s + '*'
-	return s
 
 class MatchRegex(Rule):
-	def __init__(self, regex, regex_flags=0): # re.IGNORECASE
+	def __init__(self, rex, flags=0): # re.IGNORECASE
 		super().__init__(				
-				description='Must match expression {0}'.format(regex),
+				description='Must match expression {0}'.format(rex),
 				callback=Normalize.as_str_matching_regex,
-				config={'rex':regex, 'flags':regex_flags}
+				config={'rex':rex, 'flags':flags}
 			)
 
 
@@ -151,8 +139,6 @@ class StrWithMinLen(Rule):
 				callback=Normalize.as_str_with_min_len,
 				config={'min_len':min_len}
 			)
-		if min_len > 0:
-			self._boundaries.append(_gen_str(min_len-1))
 
 
 class StrWithMaxLen(Rule):
@@ -162,8 +148,6 @@ class StrWithMaxLen(Rule):
 				callback=Normalize.as_str_with_max_len,
 				config={'max_len':max_len}
 			)
-		if max_len > 0 and max_len < (10*1024*1024):
-			self._boundaries.append(_gen_str(max_len+1))
 
 
 class StrWithinInclusiveLenRange(Rule):
@@ -173,10 +157,6 @@ class StrWithinInclusiveLenRange(Rule):
 				callback=Normalize.as_str_within_inclusive_len_range,
 				config={'max_len':max_len, 'min_len':min_len}
 			)
-		if min_len > 0:
-			self._boundaries.append(_gen_str(min_len-1))
-		if max_len > 0 and max_len < (10*1024*1024):
-			self._boundaries.append(_gen_str(max_len+1))
 
 class ValueInSet(Rule):
 	def __init__(self, values):
@@ -193,6 +173,15 @@ class ValueInSet(Rule):
 				config={'values':values}
 			)
 
+	def get_config(self):
+		config = {}
+		config.update(self._config)
+		values = config['values']
+		if isinstance(values, set):
+			config['values'] = list(config['values'])
+		return config
+
+
 	def list_values(self):
 		result = list(self.get_config()['values'])
 		result.sort()
